<?php

namespace App\Jobs;

use App\Events\VideoConversionProgress;
use Illuminate\Bus\Batchable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Foundation\Queue\Queueable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\Log;
use Symfony\Component\Process\Process;

class ConvertVideoForEachQuality implements ShouldQueue
{
    use Batchable, Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    private string $outputPath;
    private string $quality;
    private string $finalVideo;
    private string $width;
    private string $height;
    private string $bitRate;
    private string $baseName;


    /**
     * Create a new job instance.
     */
    public function __construct(string $outputPath, string $quality, string $final_video, string $width, string $height, string $bit_rate, string $baseName)
    {
        $this->outputPath = $outputPath;
        $this->quality = $quality;
        $this->finalVideo = $final_video;
        $this->width = $width;
        $this->height = $height;
        $this->bitRate = $bit_rate;
        $this->baseName = $baseName;
    }

    /**
     * Execute the job.
     */
    public function handle(): void
    {
        if($this->batch()?->canceled()){
            return;
        }
        $batchId = $this->batch()?->id ?? 'default';
        $outputFile = "{$this->outputPath}/{$this->baseName}_{$this->quality}.mp4";
        $command = [
            'ffmpeg',
            '-i', $this->finalVideo,
            '-vf', "scale={$this->width}:{$this->height}",
            '-b:v', $this->bitRate,
            '-c:a', 'aac',
            '-b:a', '128k',
            '-y', $outputFile
        ];
        $process = new Process($command);
        $process->setTimeout(null);
        $duration = $this->getVideoDuration($this->finalVideo);
        $process->run(function ($type, $buffer) use ($duration, $batchId, $outputFile) {
            if (Process::ERR === $type) {
                if (preg_match('/time=(\d+:\d+:\d+\.\d+)/', $buffer, $matches)) {
                    $currentTime = $this->timeToSeconds($matches[1]);
                    $progress = $duration > 0 ? round(($currentTime / $duration) * 100, 2) : 0;
                    // Update cache
                    $currentProgress = Cache::get("progress-{$batchId}", []);
                    $currentProgress[$this->quality] = min($progress, 100);
                    Cache::put("progress-{$batchId}", $currentProgress, 3600);
                    // Broadcast event
                    event(new VideoConversionProgress(
                        $batchId,
                        $this->quality,
                        $currentProgress[$this->quality],
                        asset("processed/{$this->baseName}_{$this->quality}.mp4")
                    ));
                }
            }
        });
        // Final progress update
        $currentProgress = Cache::get("progress-{$batchId}", []);
        $currentProgress[$this->quality] = 100;
        Cache::put("progress-{$batchId}", $currentProgress, 3600);
        event(new VideoConversionProgress($batchId, $this->quality, 100, asset("processed/{$this->baseName}_{$this->quality}.mp4")));
//        shell_exec("ffmpeg -i {$this->finalVideo} -vf scale={$this->width}:{$this->height} -b:v {$this->bitRate} -c:a aac -b:a 128k -y {$outputFile} && wait");
    }

    private function getVideoDuration($videoPath)
    {
        $process = new Process(['ffprobe', '-v', 'error', '-show_entries', 'format=duration', '-of', 'default=noprint_wrappers=1:nokey=1', $videoPath]);
        $process->run();
        return (float) trim($process->getOutput());
    }

    private function timeToSeconds($time)
    {
        $parts = explode(':', $time);
        $hours = (int)$parts[0];
        $minutes = (int)$parts[1];
        $seconds = (float)$parts[2];
        return $hours * 3600 + $minutes * 60 + $seconds;
    }

}
